#!/bin/bash
# Copyright (c) 2000-2016 Synology Inc. All rights reserved.

PKG_NAME="GitLab"
BACKUP_CONFIG="/usr/syno/etc/packages/Docker-GitLab/config"
MIGRATE_VERSION="0050"

NeedRestore()
{
	if [ -f "$BACKUP_CONFIG" ]; then
		return 0
	else
		return 1
	fi
}

NeedMigrateDB()
{
	local version="$1"
	if [ -z "$version" ] || [ "$version" -lt "$MIGRATE_VERSION" ]; then
		return 0
	fi
	return 1
}

GetDBName()
{
	get_key_value "$BACKUP_CONFIG" DB_NAME
}

GetDBUser()
{
	#version < "9.4.4-0050" does not store db_user in config
	DB_USER=$(get_key_value "$BACKUP_CONFIG" DB_USER)
	DB_USER=${DB_USER:-gitlab_user}
	echo "$DB_USER"
}

GetDBPass()
{
	get_key_value "$BACKUP_CONFIG" DB_PASS
}

GetShare()
{
	get_key_value "$BACKUP_CONFIG" SHARE
}

GetPkgVer()
{
	get_key_value "$BACKUP_CONFIG" PKG_VER
}
wizard_db_settings="Настроить базу данных GitLab"
wizard_db_name_desc="Введите имя базы данных GitLab."
wizard_db_name_label="Имя базы данных"
wizard_m10_desc="Введите учетные данные администратора MariaDB 10 для использования настроек базы данных GitLab."
wizard_admin_acc="Учетная запись"
wizard_admin_pass="Пароль"
wizard_set_db_desc="Создайте новую эксклюзивную учетную запись базы данных для GitLab для использования существующих/новых данных."
wizard_set_data_title="Настроить GitLab"
wizard_found_backup="Выберите способ импорта данных."
wizard_decide_restore="GitLab База данных уже существует.<br>Выберите одно из следующих действий."
wizard_restore="Использовать существующие данные"
wizard_create_new="Чистая установка (все имеющиеся данные, включая файлы конфигурации и базу данных, будут удалены)."
wizard_db_user_account_desc="Учетная запись пользователя базы данных"
wizard_db_user_password_desc="Пароль пользователя базы данных"
wizard_migrate_note="Последняя версия GitLab поддерживает только MariaDB 10.<br>Система обнаружила существующую базу данных MariaDB 5.<br>На следующем этапе ваша база данных будет перенесена из MariaDB 5 в MariaDB 10.<br><br>Введите учетные данные администратора MariaDB 5 для продолжения установки."
wizard_migrate_title="Перенести базу данных GitLab"
install_title="Установить GitLab"
install_data_root_desc="Создайте папку общего доступа для хранения данных GitLab."
install_data_root_label="Имя папки общего доступа"
http_port_desc="Введите номер порта HTTP для GitLab."
http_port_label="Номер порта HTTP"
ssh_port_desc="Введите номер порта SSH для GitLab."
ssh_port_label="Номер порта SSH"
hostname_label="Имя домена"
admin_email_desc="Введите учетную запись электронной почты администратора GitLab."
admin_email_label="электронная почта"
smtp_enable_desc="Включите GitLab для отправки сообщений эл. почты посредством сервера SMTP."
smtp_enable_label="Включить SMTP"
smtp_address_desc="Укажите адрес сервера SMTP, на котором размещен GitLab."
smtp_address_label="Адрес сервера SMTP"
smtp_port_desc="Введите порт сервера SMTP."
smtp_port_label="Порт сервера SMTP"
smtp_user_desc="Укажите имя пользователя сервера SMTP."
smtp_user_label="Имя пользователя сервера SMTP"
smtp_pass_desc="Введите пароль сервера SMTP."
smtp_pass_label="Пароль сервера SMTP"
smtp_verify_desc="При подключении к GitLab проверьте сертификат SSL сервера SMTP."
smtp_verify_label="Проверьте сертификат SSL и исключите небезопасные подключения."
default_gitlab_account_desc="Учетные данные по умолчанию — root/5iveL!fe"
PageM5()
{
cat << EOF
{
	"step_title": "$wizard_migrate_title",
	"items": [{
		"type": "textfield",
		"desc": "$wizard_migrate_note",
		"subitems": [{
			"key": "wizard_m5_acc",
			"desc": "$wizard_admin_acc",
			"defaultValue": "root",
			"validator": {
				"allowBlank": false
			}
		}]
	}, {
		"type": "password",
		"subitems": [{
			"indent": 1,
			"key": "wizard_m5_pass",
			"desc": "$wizard_admin_pass"
		}]
	}]
}
EOF
}

PageM10()
{
cat << EOF
{
	"step_title": "$wizard_set_data_title",
	"items": [{
		"type": "textfield",
		"desc": "$wizard_m10_desc",
		"subitems": [{
			"key": "wizard_m10_acc",
			"desc": "$wizard_admin_acc",
			"defaultValue": "root",
			"validator": {
				"allowBlank": false
			}
		}]
	}, {
		"type": "password",
		"subitems": [{
			"indent": 1,
			"key": "wizard_m10_pass",
			"desc": "$wizard_admin_pass"
		}]
	}]
}
EOF
}
CheckRestore()
{
cat << EOF
// find constructor contains restore page
for (i = arguments[0].ownerCt.items.length-1; i >= 1; i--){
	page = arguments[0].ownerCt.items.items[i];
	if (page.headline === \"${wizard_found_backup}\"){
		// check whether user wants to restore or not
		restore = page.items.items[1].checked;
		break;
	}
}
EOF
}

FindObj()
{
cat << EOF
for (i = 0; i < arguments[0].items.length; i++) {
	item = arguments[0].items.items[i]
	if (\"${1}\" === item.itemId){
		$2 = arguments[0].items.items[i];
		break;
	}
}
EOF
}

Activate()
{
cat << EOF
	"activeate": "{
		restore = $DEFAULT_RESTORE; // user restore or not, set by is_restore
		// will set restore
		$(CheckRestore)

		if (arguments[0].headline === \"${wizard_db_settings}\") {
			create_db_multiselect = null;
			need_migrate_multiselect = null;
			old_db_textfield= null;
			drop_db_multiselect= null;
			new_db_textfield = null;
			db_user_textfield = null;
			grant_user_multiselect = null;
			mariadb_ver_textfield = null;

			$(FindObj "create_db_flag" "create_db_multiselect")
			$(FindObj "need_migrate" "need_migrate_multiselect")
			$(FindObj "pkgwizard_db_name" "new_db_textfield")
			$(FindObj "old_db_name" "old_db_textfield")
			$(FindObj "drop_db_flag" "drop_db_multiselect")
			$(FindObj "pkgwizard_db_user_account" "db_user_textfield")
			$(FindObj "grant_user_flag" "grant_user_multiselect")
			$(FindObj "mariadb_ver" "mariadb_ver_textfield")

			old_db = \"@OLD_DB@\";

			if (restore) {
				db_name = \"@OLD_DB@\";
				db_user = \"@OLD_USER@\";
			}
			else {
				db_name = \"gitlab\";
				db_user = \"gitlab_user\";
			}

			old_db_textfield.setValue(old_db);
			new_db_textfield.setValue(db_name);
			new_db_textfield.setDisabled(restore);
			db_user_textfield.setValue(db_user);
			mariadb_ver_textfield.setValue(\"$MARIADB_VER\");

			need_migrate_multiselect.setVisible(false);
			create_db_multiselect.setVisible(false);
			drop_db_multiselect.setVisible(false);
			grant_user_multiselect.setVisible(false);

		}

		if (arguments[0].items.items[0].value === \"${install_data_root_desc}\") {
			dataroot_textfield = null;
			$(FindObj "pkgwizard_dataroot" "dataroot_textfield")
			if (restore) {
				dataroot_textfield.setValue(\"@OLD_DATAROOT@\");
			}
			dataroot_textfield.setDisabled(restore);
		}
	}",
EOF
}

Deactivate()
{
cat << EOF
	"deactivate": "{
		restore = $DEFAULT_RESTORE;
		$(CheckRestore) // if no radio button, will not set restore value
		if (arguments[0].headline === \"${wizard_db_settings}\") {
			create_db_multiselect = null;
			need_migrate_multiselect = null;
			create_db_collision_textfield = null;
			old_db_textfield= null;
			drop_db_multiselect= null;
			new_db_textfield = null;
			grant_user_multiselect = null;

			$(FindObj "create_db_flag" "create_db_multiselect")
			$(FindObj "need_migrate" "need_migrate_multiselect")
			$(FindObj "create_db_collision" "create_db_collision_textfield")
			$(FindObj "old_db_name" "old_db_textfield")
			$(FindObj "drop_db_flag" "drop_db_multiselect")
			$(FindObj "pkgwizard_db_name" "new_db_textfield")
			$(FindObj "grant_user_flag" "grant_user_multiselect")
			table = {
				\"5to10\": {\"migrate\": true, \"create\": false, \"grant_user\": true, \"drop_db\": true, \"create_db_collision\": \"error\"},
				\"restore10\": {\"migrate\": false, \"create\": true, \"grant_user\": true, \"drop_db\": false, \"create_db_collision\": \"skip\"},
				\"new10\": {\"migrate\": false, \"create\": true, \"grant_user\": true, \"drop_db\": old_db_textfield.rawValue && (old_db_textfield.rawValue !== new_db_textfield.rawValue)? true : false, \"create_db_collision\": old_db_textfield.rawValue === new_db_textfield.rawValue? \"replace\" : \"error\"}
			};

			if (restore) {
				if ($NEED_MIGRATE) {
					state = \"5to10\";
				}
				else {
					state = \"restore10\";
				}
			}
			else {
				state = \"new10\";
			}

			need_migrate_multiselect.setValue(table[state][\"migrate\"]);
			create_db_multiselect.setValue(table[state][\"create\"]);
			grant_user_multiselect.setValue(table[state][\"grant_user\"]);
			drop_db_multiselect.setValue(table[state][\"drop_db\"]);
			create_db_collision_textfield.setValue(table[state][\"create_db_collision\"]);
		}
}",
EOF
}

ApplyDBInfo()
{
	local page_db="$1"

	sed "s/@OLD_DB@/$DB_NAME/g;s/@OLD_USER@/$DB_USER/g;s/@OLD_DATAROOT@/$OLD_DATAROOT/g" <<< "$page_db"
}

PageDB()
{
	local page_db=$(cat << EOF
{
	"step_title": "$wizard_db_settings",
	$(Activate)
	$(Deactivate)
	"items": [{
		"type": "textfield",
		"desc": "$wizard_set_db_desc",
		"subitems": [{
			"key": "pkgwizard_db_name",
			"desc": "$wizard_db_name_label",
			"disabled": true,
			"validator": {
				"allowBlank": false
			}
		}]
	}, {
		"type": "textfield",
		"subitems": [{
			"indent": 1,
			"key": "pkgwizard_db_user_account",
			"desc": "$wizard_db_user_account_desc",
			"validator": {
				"allowBlank": false
			}
		}]
	}, {
		"type": "password",
		"subitems": [{
			"indent": 1,
			"key": "pkgwizard_db_user_password",
			"desc": "$wizard_db_user_password_desc"
		}]
	}, {
		"type": "textfield",
		"subitems": [{
			"key": "create_db_collision",
			"desc": "drop or skip",
			"defaultValue": "skip",
			"hidden": true
		}]
	}, {
		"type": "multiselect",
		"subitems": [{
			"key": "need_migrate",
			"desc": "migrate or not",
			"hidden": true
		}]
	}, {
		"type": "multiselect",
		"subitems": [{
			"key": "create_db_flag",
			"desc": "create db or not",
			"hidden": true
		}]
	}, {
		"type": "textfield",
		"subitems": [{
			"key": "old_db_name",
			"desc": "old db name",
			"hidden": true
		}]
	}, {
		"type": "multiselect",
		"subitems": [{
			"key": "drop_db_flag",
			"desc": "drop old db or not",
			"hidden": true
		}]
	}, {
		"type": "multiselect",
		"subitems": [{
			"key": "grant_user_flag",
			"desc": "must grant user if exist wizard",
			"defaultVaule": true,
			"hidden": true
		}]
	}, {
		"type": "textfield",
		"subitems": [{
			"key": "mariadb_ver",
			"desc": "mariadb version",
			"defaultVaule": "$MARIADB_VER",
			"hidden": true
		}]
	}]
}
EOF
)
	ApplyDBInfo "$page_db"
}
main()
{
	local upgrade_page=""
	DEFAULT_RESTORE=true
	NEED_MIGRATE=false
	MARIADB_VER="m10"
	DB_NAME=$(GetDBName)
	DB_USER=$(GetDBUser)

	local version="$(echo "$SYNOPKG_OLD_PKGVER" | cut -d- -f2)"
	if NeedMigrateDB "$version"; then # check if needing to migrate MariaDB 5
		NEED_MIGRATE=true
		MARIADB_VER="m5"
		upgrade_page="$(PageM5), $(PageM10), $(PageDB)"
	fi
	echo "[$upgrade_page]" > "${SYNOPKG_TEMP_LOGFILE}"

	return 0
}

main "$@"
